package controllers

import (
	"errors"
	"net/http"
	"spaceofcheng/services/models"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
	"strings"

	"github.com/astaxie/beego"
)

// Authenticate 权限验证
func (c *BaseController) authenticate() {
	// 默认 service
	serv := service.NewSysServ(c.Context)

	// 客户端类型
	clientType := utils.GetClientType(c.Ctx.Request)

	switch clientType {
	case utils.ClientAdmin:
		c.authPCAdmin(serv)
	case utils.ClientWechat:
		c.authWechat(serv)
	default:
		c.ResponseError(
			errors.New("暂不支持的 API 场景"),
			http.StatusBadRequest,
		)
	}
}

// authPCAdmin PC管理端
func (c *BaseController) authPCAdmin(serv *service.SysServ) {
	if !c.needAuth() {
		return
	}

	// 用户ID
	userID := ""
	userIDRaw := c.GetSession(SNUserID)
	if userIDRaw != nil {
		userID = userIDRaw.(string)
	}

	if userID == "" {
		c.ResponseError(
			errors.New("用户未登录"),
			http.StatusUnauthorized,
		)
	}

	if err := serv.SetUserProfile(userID); err != nil {
		utils.LogError(err.Error())

		c.ResponseError(
			errors.New("内部错误, 请重试"),
			http.StatusInternalServerError,
		)
	}

	// 设置 Session
	c.SetSession(SNUserID, userID)
	// c.SetSession(SNUserType, userType)
}

func (c *BaseController) authWechat(serv *service.SysServ) {
	// 用户类型, 此处与数据库定义的不是一个概念
	// 此处主要用来区分是 系统用户, 还是 系统客户
	userType := ""
	userTypeRaw := c.GetSession(SNUserType)
	if userTypeRaw != nil {
		userType = userTypeRaw.(string)
	}

	if userType == "" {
		userType = models.USERMAP_CUSTOMER
		identify := c.GetString("identify_type")
		if identify == "admin" {
			userType = models.USERMAP_USER
		}
	}

	// 用户类型不同, 此处ID代表的内容也是不一样的
	userID := ""
	userIDRaw := c.GetSession(SNUserID)
	if userIDRaw != nil {
		userID = userIDRaw.(string)
	}

	// 用户微信信息
	var wxDetail map[string]interface{}
	detailRaw := c.GetSession("wechat_user")
	if detailRaw != nil {
		wxDetail = detailRaw.(map[string]interface{})
	}

	if userID == "" && wxDetail == nil {
		wxDetail = c.getWechatDetail()
		c.SetSession("wechat_user", wxDetail)
	}

	// 用户映射
	// 如果映射不存在, 则新建映射
	userMap, err := serv.CheckWechatUserMapping(wxDetail, userType)
	if err != nil {
		c.ResponseError(
			utils.LogError("获取人员映射账户失败: "+err.Error()),
			http.StatusInternalServerError,
		)

		c.Context.Set("userMap", *userMap)
	}

	if !c.needAuth() {
		return
	}

	// 未登录或者绑定, 返回 401
	if userID == "" && userMap.UserId == "" {
		wxConf, ok := c.Configer[WeChatConf]
		if ok {
			c.ResponseData(
				map[string]interface{}{
					"appid": wxConf.String("appid"),
				},
				errors.New("用户未登录"),
				http.StatusUnauthorized,
			)
		} else {
			c.ResponseError(
				errors.New("用户未登录"),
				http.StatusUnauthorized,
			)
		}
	}

	// 人员微信账号绑定错误
	if userID != "" && userMap.UserId != "" &&
		(userID != userMap.UserId || userType != userMap.UserType) {
		c.ResponseError(
			errors.New("系统内部数据错误, 人员绑定不正确"),
			http.StatusInternalServerError,
		)
	}

	if userID == "" {
		userID = userMap.UserId
	}

	if userType == models.USERMAP_USER {
		if err := serv.SetUserProfile(userID); err != nil {
			utils.LogError(err.Error())

			c.ResponseError(
				errors.New("内部错误, 请重试"),
				http.StatusInternalServerError,
			)
		}
	} else {
		if err := serv.SetCustomer(userID); err != nil {
			utils.LogError(err.Error())

			c.ResponseError(
				errors.New("内部错误, 请重试"),
				http.StatusInternalServerError,
			)
		}
	}

	// 设置 Session
	c.SetSession(SNUserID, userID)
	c.SetSession(SNUserType, userType)
}

func (c *BaseController) needAuth() bool {
	route := c.Ctx.Input.URL()
	apiPrefix := beego.AppConfig.String("api::prefix")
	guestAPI := beego.AppConfig.String("api::guest")

	if strings.Index(route, apiPrefix+guestAPI) > -1 {
		return false
	}

	return true
}

// getWechatDetail 获取微信个人信息详情
func (c *BaseController) getWechatDetail() map[string]interface{} {
	// 微信 code
	code := c.GetString("code")
	if code == "" {
		c.ResponseError(
			errors.New("没有接收到微信 code"),
			http.StatusBadRequest,
		)
	}

	usr, err := utils.GetWxUserInfo(code)
	if err != nil {
		c.ResponseError(
			utils.LogError("获取微信个人信息失败: "+err.Error()),
			http.StatusInternalServerError,
		)
	}

	return usr
}
