package customer

import (
	"encoding/json"
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/customer"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/events"
	"spaceofcheng/services/utils"
	"strconv"
)

// CustomerServ 系统处理
type CustomerServ struct {
	ctx *utils.Context
	dao *customer.CustomerDAO
}

// NewCustomerServ 初始化
func NewCustomerServ(ctx *utils.Context) *CustomerServ {
	return &CustomerServ{
		ctx: ctx,
		dao: customer.NewCustomerDAO(ctx),
	}
}

// GetCustWithWXList 获取客户列表
func (s *CustomerServ) GetCustWithWXList(phone string, page ...int) ([]customer.CustWithWX, int64, error) {
	return s.dao.CustWithWXList(phone, page...)
}

// GetCustWithWXByID 获取客户列表
func (s *CustomerServ) GetCustWithWXByID(custID string) (*customer.CustWithWX, error) {
	cust, err := s.dao.GetCustWithWXByID(custID)
	if err != nil {
		return nil, err
	}

	if cust.Status != models.STATUS_NORMAL {
		return nil, errors.New("用户状态异常")
	}

	return cust, nil
}

// GetCustUserByID 获取用户信息(自定义内容)
func (s *CustomerServ) GetCustUserByID(userID string) (*UserProfile, error) {
	user, err := s.dao.GetUserByID(userID)
	if err != nil {
		return nil, err
	}

	if user.Status != models.STATUS_NORMAL {
		return nil, errors.New("用户状态异常")
	}

	userProfile := UserProfile{
		UserId:     user.UserId,
		UserName:   user.UserName,
		RealName:   user.RealName,
		Sex:        user.Sex,
		WorkNo:     user.WorkNo,
		Email:      user.Email,
		Phone:      user.Phone,
		Headimgurl: user.Headimgurl,
	}

	return &userProfile, nil
}

// BindWechat 绑定微信用户
func (s *CustomerServ) BindWechat(userMap *model.TaUserMapping, phone, caseID, userID string) (*model.TaCustomer, error) {
	cust, err := s.dao.GetCustomerByPhone(phone)
	if err != nil {
		return nil, err
	}

	// 用户不存在, 则新增
	if cust == nil {
		newCust, err := s.SaveNewCustomer(phone, caseID, userID)
		if err != nil {
			return nil, err
		}

		cust = newCust
	}

	if cust.Status != models.STATUS_NORMAL {
		return nil, errors.New("用户状态不正确")
	}

	if cust.CustomerId == userMap.UserId {
		return cust, nil
	}

	if userMap.UserId != "" {
		return nil, errors.New("内部错误, 用户映射信息有误")
	}

	userMap.UserId = cust.CustomerId
	if err := s.dao.UpdateUserMapping(userMap, []string{"user_id"}); err != nil {
		utils.LogError("更新用户映射信息失败:" + err.Error())
		return nil, errors.New("映射用户信息失败")
	}

	// triggerRegiteEvent 触发注册事件
	utils.EmitEvent(events.EvtRegiste, *userMap)

	return cust, nil
}

// SaveNewCustomer 新增客户
func (s *CustomerServ) SaveNewCustomer(phone, caseID, userID string) (*model.TaCustomer, error) {
	// 校验案场信息
	cs, err := s.dao.GetCaseByID(caseID)
	if err != nil {
		return nil, err
	}

	if cs == nil || cs.Status == models.STATUS_DEL {
		return nil, errors.New("系统无该推荐案场")
	}

	// 校验推荐人
	user, err := s.dao.GetUserByID(userID)
	if err != nil {
		return nil, err
	}

	if user == nil || user.Status == models.STATUS_DEL {
		return nil, errors.New("系统无该推荐人")
	}

	// 微信信息
	userMap := s.ctx.Get("userMap").(model.TaUserMapping)
	wxInfo := make(map[string]interface{})
	if err := json.Unmarshal([]byte(userMap.AccountInfo), &wxInfo); err != nil {
		utils.LogError(err.Error())
		return nil, errors.New("校验用户微信信息失败")
	}

	nickyName := wxInfo["nickname"].(string)
	sex := wxInfo["sex"].(string)
	headimgurl := wxInfo["headimgurl"].(string)

	sexInt, _ := strconv.Atoi(sex)
	if sexInt == 0 {
		sexInt = 1 // 默认男性
	}

	cust := model.TaCustomer{
		OrgId:         cs.OrgId,
		CustomerName:  nickyName,
		Name:          nickyName,
		Sex:           sexInt,
		Headimgurl:    headimgurl,
		RecommendId:   user.UserId,
		RecommendName: user.RealName,
		RecommendCase: cs.CaseId,
	}

	if err := s.dao.SaveCustomer(&cust); err != nil {
		utils.LogError("更新客户信息失败: " + err.Error())
		return nil, errors.New("更新客户信息失败")
	}

	return &cust, nil
}
